/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.felix.ipojo.plugin;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import org.apache.felix.ipojo.manipulator.Pojoization;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;

/**
 * Package an OSGi jar "bundle" as an "iPOJO bundle".
 * 
 * @author <a href="mailto:dev@felix.apache.org">Felix Project Team</a>
 * @version $Rev$, $Date$
 * @goal ipojo-bundle
 * @phase package
 * @requiresDependencyResolution runtime
 * @description manipulate an OSGi bundle jar to build an iPOJO bundle
 */
public class ManipulatorMojo extends AbstractMojo {

    /**
     * The directory for the generated JAR.
     * 
     * @parameter expression="${project.build.directory}"
     * @required
     */
    private String m_buildDirectory;

    /**
     * The directory containing generated classes.
     * 
     * @parameter expression="${project.build.outputDirectory}"
     * @required
     * @readonly
     */
    private File m_outputDirectory;

    /**
     * The name of the generated JAR file.
     * 
     * @parameter alias="jarName" expression="${project.build.finalName}"
     * @required
     */
    private String m_jarName;
    
    /**
     * Metadata file location.
     * @parameter alias="metadata" default-value="metadata.xml"
     */
    private String m_metadata;

    /**
     * The Maven project.
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject m_project;

    /**
     * Project types which this plugin supports.
     * @parameter
     */
    private List m_supportedProjectTypes = Arrays.asList(new String[]{"bundle"});

    /**
     * Ignore annotations parameter.
     * @parameter alias="ignoreAnnotations" default-value="false"
     */
    private boolean m_ignoreAnnotations;

    protected MavenProject getProject() {
        return this.m_project;
    }

    /**
     * Execute method : launch the pojoization.
     * @throws MojoExecutionException : an exception occurs.
     * @throws MojoFailureException : an failure occurs.
     * @see org.apache.maven.plugin.AbstractMojo#execute()
     */
    public void execute() throws MojoExecutionException, MojoFailureException {
        // ignore project types not supported, useful when the plugin is configured in the parent pom
        if (!this.m_supportedProjectTypes.contains(this.getProject().getArtifact().getType())) {
            this.getLog().debug("Ignoring project " + this.getProject().getArtifact() + " : type " + this.getProject().getArtifact().getType() + " is not supported by ipojo plugin, supported types are " + this.m_supportedProjectTypes);
            return;
        }

        getLog().info("Start bundle manipulation");
        // Get metadata file
        File meta = new File(m_outputDirectory + "/" + m_metadata);
        getLog().info("Metadata File : " + meta.getAbsolutePath());
        if (!meta.exists()) {
            // Verify if annotations are ignored
            if (m_ignoreAnnotations) {
                getLog().info("No metadata file found - ignore annotations");
                return;
            } else {
                getLog().info("No metadata file found - try to use only annotations");
                meta = null;
            }
        }

        // Get input bundle
        File in = new File(m_buildDirectory + "/" + m_jarName + ".jar");
        getLog().info("Input Bundle File : " + in.getAbsolutePath());
        if (!in.exists()) {
            throw new MojoExecutionException("the specified bundle file does not exists");
        }
        
        File out = new File(m_buildDirectory + "/_out.jar");
        
        Pojoization pojo = new Pojoization();
        if (!m_ignoreAnnotations) { pojo.setAnnotationProcessing(); }
        pojo.pojoization(in, out, meta);
        for (int i = 0; i < pojo.getWarnings().size(); i++) {
            getLog().warn((String) pojo.getWarnings().get(i));
        }
        if (pojo.getErrors().size() > 0) { throw new MojoExecutionException((String) pojo.getErrors().get(0)); }
        in.delete();
        out.renameTo(in);
        getLog().info("Bundle manipulation - SUCCESS");
    }

}
